@@ -1,65 +1,68 @@
 from plotext._default import default_figure_class
 from plotext._monitor import monitor_class
-import plotext._utility.doc as doc
+from plotext._matrix import join_matrices
+from plotext._date import date_class
+import plotext._doc as doc
+from time import time as _time
 import plotext._utility as ut
 from time import time
 import os
 
+# A figure is a general container of either a plot (called monitor) or another figure, when subplots are nested
+# This creates a hierarchy of figures, where master is the main/initial global figure, and parent is the figure containing (or above) the one considered
+# The active figure is the one that can be accessed with further plotext commands - like plot(), limitsize() etc ..
+# If a figure has no sub figures, then it is used for plotting, otherwise its sub figures are checked
+
 class _figure_class():
+    
     def __init__(self, master = None, parent = None):
-        self._set_family(master, parent) 
-        self.default = default_figure_class() # default values
-        self._time = None # computational time
+        self._set_family(master, parent) # it sets master, parent and active figure
+        self.default = default_figure_class() # default values of figure class
+        self.date = date_class()
 
-        self._set_size(None, None) # no initial size
-        self._limit_master_size(True, True) if self._is_master else None # limit size initially for master
-        self._set_terminal_size(*ut.terminal_size()) if self._is_master else None # get terminal size for master
-        self._set_master_size() if self._is_master else None # set master size to terminal
+        self._set_size(None, None) # no initial size for general figure
+        self.max_or_min = max # in a matrix of subplots the maximum height/width is considered (by default) for each row/column
         
-        self._set_slots_max(*self._master._size) # sets the maximum number of subplots in the current figure
-        self._set_slots(0, 0) # no subplots, the current figure is used as plotting monitor
-        self._set_subplots() 
+        self.monitor = monitor_class() if self._is_master else self._parent.monitor.copy() # each figure has a monitor for plotting; which by default is deep copied from its parent figure monitor, so that a figure with multiple sub plots can have easily the same plot and plot settings and preferences (without rewriting code)
+        self.monitor.set_date(self.date) # to make sure that the date settings of a figure are the same for its subplots
+
+        self._set_master() if self._is_master else None # it sets the master figure size and other utilities 
+
+        self._set_slots_max(*self._master._size) # sets the maximum number of sub figures in the current figure (from master figure size)
+        self.subplots(0, 0) # no sub figures added by default, so that the current figure is used for plotting
         
-        self.monitor = monitor_class() if self._is_master else self._parent.monitor.copy() # each figure has a monitor for plotting; which by default is copied from the parent figure monitor so that all plots can be duplicated in each subplot without rewriting code 
-        self.monitor.size = self._size
-        self.date = self.monitor.date
-
-        if self._is_master:
-            self.dummy = _figure_class(self._master, self._parent) # the master has a dumm container for subplots that do not actually exist (anymore due to change of size) 
-
     def _set_family(self, master = None, parent = None):
         self._parent = self if parent is None else parent # the figure just above this one
         self._master = self if master is None else master # the figure above all others
         self._is_master = self is self._master
-        self._active = self if self._is_master else self._master._active # the active figure
-        self._take_max()
+        self._active = self if self._is_master else self._master._active # the active figure, such that further plotting or settings commands refer to it
 
-    def main(self): # returns the master figure
-        self._master._active = self._master
-        return self._master
+    def _set_master(self):
+        self._limit_size(True, True)  # limit size (only available for master, as sub figures are by default limited by parent figure)
+        self._set_terminal_size(*ut.terminal_size()) # get and set terminal size
+        self._set_master_size()  # set master size to terminal
+        self._time = None # computational time of show() method, only available for global figure
+        self._dummy = _figure_class(self._master, self._master) # the master has a dumm container for subplots that do not actually exist (anymore due to change of size)
+        self._master.monitor.set_size(self._size)
+        self._dummy.monitor.set_size(self._size)
+        self._set_interactive() # if to make the final figure interactive: every command gets directly printed
 
 ##############################################
 ###########    Size Functions    #############
 ##############################################
 
-    def _limit_master_size(self, width = None, height = None):
-        self._limit_width = self.default.limit_width if width is None else bool(width)
-        self._limit_height = self.default.limit_height if height is None else bool(height)
-        self._limit = [self._limit_width, self._limit_height]
+    def _set_interactive(self, interactive = None):
+        self._interactive = self.default.interactive if interactive is None else bool(interactive)
 
     def _set_size(self, width = None, height = None):
         self._width = None if width is None else int(width)
         self._height = None if height is None else int(height)
         self._size = [self._width, self._height]
 
-    def plot_size(self, width = None, height = None):
-        width = self._width if width is None else width
-        height = self._height if height is None else height
-        self._set_size(width, height)
-        self._set_master_size() if self._is_master else None
-        self.monitor.size = self._size
-        
-    plotsize = plot_size
+    def _limit_size(self, width = None, height = None):
+        self._limit_width = self.default.limit_width if width is None else bool(width)
+        self._limit_height = self.default.limit_height if height is None else bool(height)
+        self._limit = [self._limit_width, self._limit_height]
 
     def _set_terminal_size(self, width = None, height = None):
         self._width_term = self.default._width_term if width is None else width
@@ -90,17 +93,12 @@ class _figure_class():
         self._Cols = list(range(1, self._cols + 1))
         self._slots = [self._rows, self._cols]
         self._no_plots = 0 in self._slots #or self._is_master
-
+        
     def _set_subplots(self):
         self.subfig = [[_figure_class(self._master, self._parent) for col in self._Cols] for row in self._Rows]
         
-    def subplots(self, rows = None, cols = None):
-        self._set_slots(rows, cols)
-        self._set_subplots()
-        return self
-
     def _get_subplot(self, row = None, col = None):
-        return self.subfig[row - 1][col - 1] if row in self._Rows and col in self._Cols else self._master.dummy
+        return self.subfig[row - 1][col - 1] if row in self._Rows and col in self._Cols else self._master._dummy
 
     def subplot(self, row = None, col = None):
         active = self._get_subplot(row, col)
@@ -108,6 +106,11 @@ class _figure_class():
         self._master._active = active
         return self._master._active
 
+    def subplots(self, rows = None, cols = None):
+        self._set_slots(rows, cols)
+        self._set_subplots()
+        return self
+
 ##############################################
 #######    External Set Functions    #########
 ##############################################
@@ -121,11 +124,11 @@ class _figure_class():
     def ylabel(self, label = None, yside = None):
         self.monitor.set_ylabel(label = label, yside = yside) if self._no_plots else [[self._get_subplot(row, col).ylabel(label = label, yside = yside) for col in self._Cols] for row in self._Rows]
 
-    def xlim(self, lower = None, upper = None, xside = None):
-        self.monitor.set_xlim(lower = lower, upper = upper, xside = xside) if self._no_plots else [[self._get_subplot(row, col).xlim(lower = lower, upper = upper, xside = xside) for col in self._Cols] for row in self._Rows]
+    def xlim(self, left = None, right = None, xside = None):
+        self.monitor.set_xlim(left = left, right = right, xside = xside) if self._no_plots else [[self._get_subplot(row, col).xlim(left = left, right = right, xside = xside) for col in self._Cols] for row in self._Rows]
 
-    def ylim(self, left = None, right = None, yside = None):
-        self.monitor.set_ylim(left = left, right = right, yside = yside) if self._no_plots else [[self._get_subplot(row, col).ylim(left = left, right = right, yside = yside) for col in self._Cols] for row in self._Rows]
+    def ylim(self, lower = None, upper = None, yside = None):
+        self.monitor.set_ylim(lower = lower, upper = upper, yside = yside) if self._no_plots else [[self._get_subplot(row, col).ylim(lower = lower, upper = upper, yside = yside) for col in self._Cols] for row in self._Rows]
         
     def xscale(self, scale = None, xside = None):
         self.monitor.set_xscale(scale = scale, xside = xside) if self._no_plots else [[self._get_subplot(row, col).xscale(scale = scale, xside = xside) for col in self._Cols] for row in self._Rows]
@@ -145,6 +148,12 @@ class _figure_class():
     def yfrequency(self, frequency = None, yside = None):
         self.monitor.set_yfrequency(frequency = frequency, yside = yside) if self._no_plots else [[self._get_subplot(row, col).yfrequency(frequency = frequency, yside = yside) for col in self._Cols] for row in self._Rows]
 
+    def xreverse(self, reverse = None, xside = None):
+        self.monitor.set_xreverse(reverse = reverse, xside = xside) if self._no_plots else [[self._get_subplot(row, col).xreverse(reverse = reverse, xside = xside) for col in self._Cols] for row in self._Rows]
+
+    def yreverse(self, reverse = None, yside = None):
+        self.monitor.set_yreverse(reverse = reverse, yside = yside) if self._no_plots else [[self._get_subplot(row, col).yreverse(reverse = reverse, yside = yside) for col in self._Cols] for row in self._Rows]
+
     def xaxes(self, lower = None, upper = None):
         self.monitor.set_xaxes(lower = lower, upper = upper) if self._no_plots else [[self._get_subplot(row, col).xaxes(lower = lower, upper = upper) for col in self._Cols] for row in self._Rows]
 
@@ -188,6 +197,10 @@ class _figure_class():
         self.monitor.clear_color() if self._no_plots else [[self._get_subplot(row, col).clear_color() for col in self._Cols] for row in self._Rows]
     clc = clear_color
 
+    def clear_terminal(self, lines = None):
+        ut.clear_terminal(lines = lines)
+    clt = clear_terminal
+
 ##############################################
 ###########    Plot Functions    #############
 ##############################################
@@ -203,29 +216,26 @@ class _figure_class():
 
     def candlestick(self, dates, data, orientation = None, colors = None, label = None):
         self.monitor.draw_candlestick(dates, data, orientation = orientation, colors = colors, label = label) if self._no_plots else [[self._get_subplot(row, col).candlestick(dates, data, orientation = orientation, colors = colors, label = label) for col in self._Cols] for row in self._Rows]
+        
+    def bar(self, *args, xside = None, yside = None, marker = None, color = None, fill = None, width = None, orientation = None, label = None, minimum = None, reset_ticks = None):
+        self.monitor.draw_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum, reset_ticks = reset_ticks) if self._no_plots else [[self._get_subplot(row, col).bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum, reset_ticks = reset_ticks) for col in self._Cols] for row in self._Rows]
 
-    def bar(self, *args, xside = None, yside = None, marker = None, color = None, fill = None, width = None, orientation = None, label = None, minimum = None):
-        self.monitor.draw_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum) if self._no_plots else [[self._get_subplot(row, col).bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum) for col in self._Cols] for row in self._Rows]
+    def multiple_bar(self, *args, xside = None, yside = None, marker = None, color = None, fill = None, width = None, orientation = None, label = None, minimum = None, reset_ticks = None):
+        self.monitor.draw_multiple_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum, reset_ticks = reset_ticks) if self._no_plots else [[self._get_subplot(row, col).multiple_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum, reset_ticks = reset_ticks) for col in self._Cols] for row in self._Rows]
     
-    def multiple_bar(self, *args, xside = None, yside = None, marker = None, color = None, fill = None, width = None, orientation = None, label = None, minimum = None):
-        self.monitor.draw_multiple_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum) if self._no_plots else [[self._get_subplot(row, col).multiple_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum) for col in self._Cols] for row in self._Rows]
-    
-    def stacked_bar(self, *args, xside = None, yside = None, marker = None, color = None, fill = None, width = None, orientation = None, label = None, minimum = None):
-        self.monitor.draw_stacked_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum) if self._no_plots else [[self._get_subplot(row, col).stacked_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum) for col in self._Cols] for row in self._Rows]
+    def stacked_bar(self, *args, xside = None, yside = None, marker = None, color = None, fill = None, width = None, orientation = None, label = None, minimum = None, reset_ticks = None):
+        self.monitor.draw_stacked_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum, reset_ticks = reset_ticks) if self._no_plots else [[self._get_subplot(row, col).stacked_bar(*args, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum, reset_ticks = reset_ticks) for col in self._Cols] for row in self._Rows]
 
     def hist(self, data, bins = None, norm = None, xside = None, yside = None, marker = None, color = None, fill = None, width = None, orientation = None, label = None, minimum = None):
         self.monitor.draw_hist(data, bins = bins, norm = norm, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum) if self._no_plots else [[self._get_subplot(row, col).hist(data, bins = bins, norm = norm, xside = xside, yside = yside, marker = marker, color = color, fill = fill, width = width, orientation = orientation, label = label, minimum = minimum) for col in self._Cols] for row in self._Rows]
 
-    def matrix_plot(self, matrix, marker = None, style = None, fast = False):
-        self.monitor.draw_matrix(matrix, marker = marker, style = style, fast = fast) if self._no_plots else [[self._get_subplot(row, col).matrix_plot(matrix, marker = marker, style = style, fast = fast) for col in self._Cols] for row in self._Rows]
-    
-    def image_plot(self, path, marker = None, style = None, grayscale = False, fast = False):
-        self.monitor.draw_image(path, marker = marker, style = style, grayscale = grayscale, fast = fast) if self._no_plots else [[self._get_subplot(row, col).image_plot(path, marker = marker, style = style, grayscale = grayscale, fast = fast) for col in self._Cols] for row in self._Rows]
-
 ##############################################
 ###########    Plotting Tools    #############
 ##############################################
 
+    def error(self, *args, xerr = None, yerr = None, xside = None, yside = None, color = None, label = None):
+        self.monitor.draw_error(*args, xerr = xerr, yerr = yerr, xside = xside, yside = yside, color = color, label = label) if self._no_plots else [[self._get_subplot(row, col).error(*args, xerr = xerr, yerr = yerr, xside = xside, yside = yside, color = color, label = label) for col in self._Cols] for row in self._Rows]
+
     def event_plot(self, data, orientation = None, marker = None, color = None, side = None):
         self.monitor.draw_event_plot(data, orientation = orientation, marker = marker, color = color, side = side) if self._no_plots else [[self._get_subplot(row, col).event_plot(data, orientation = orientation, marker = marker, color = color, side = side) for col in self._Cols] for row in self._Rows]
     eventplot = event_plot
@@ -238,94 +248,39 @@ class _figure_class():
         self.monitor.draw_horizontal_line(coordinate, color = color, yside = yside) if self._no_plots else [[self._get_subplot(row, col).horizontal_line(coordinate, color = color, yside = yside) for col in self._Cols] for row in self._Rows]     
     hline = horizontal_line
 
-    def text(self, text, x, y, xside = None, yside = None, color = None, style = None, alignment = None):
-        self.monitor.draw_text(text, x, y, xside = xside, yside = yside, color = color, style = style, alignment = alignment) if self._no_plots else [[self._get_subplot(row, col).text(text, x, y, xside = xside, yside = yside, color = color, style = style, alignment = alignment) for col in self._Cols] for row in self._Rows]
+    def text(self, text, x, y, xside = None, yside = None, color = None, background = None, style = None, orientation = None, alignment = None):
+        self.monitor.draw_text(text, x, y, xside = xside, yside = yside, color = color, background = background, style = style, orientation = orientation, alignment = alignment) if self._no_plots else [[self._get_subplot(row, col).text(text, x, y, xside = xside, yside = yside, color = color, background = background, style = style, orientation = orientation, alignment = alignment) for col in self._Cols] for row in self._Rows]
+
+    def rectangle(self, x = None, y = None, xside = None, yside = None, lines = None, marker = None, color = None, fill = None, label = None):
+        self.monitor.draw_rectangle(x = x, y = y, xside = xside, yside = yside, lines = lines, marker = marker, color = color, fill = fill, label = label) if self._no_plots else [[self._get_subplot(row, col).rectangle(x = x, y = y, xside = xside, yside = yside, lines = lines, marker = marker, color = color, fill = fill, label = label) for col in self._Cols] for row in self._Rows]
+
+    def polygon(self, x = None, y = None, radius = None, sides = None, xside = None, yside = None, lines = None, marker = None, color = None, fill = None, label = None):
+        self.monitor.draw_polygon(x = x, y = y, radius = radius, sides = sides, xside = xside, yside = yside, lines = lines, marker = marker, color = color, fill = fill, label = label) if self._no_plots else [[self._get_subplot(row, col).polygon(x = x, y = y, radius = radius, sides = sides, xside = xside, yside = yside, lines = lines, marker = marker, color = color, fill = fill, label = label) for col in self._Cols] for row in self._Rows]
+
+    def confusion_matrix(self, actual, predicted, labels = None, color = None, style = None):
+        self.monitor.draw_confusion_matrix(actual, predicted, labels = labels, color = color, style = style) if self._no_plots else [[self._get_subplot(row, col).confusion_matrix(actual, predicted, labels = labels, color = color, style = style) for col in self._Cols] for row in self._Rows]
+
+    cmatrix = confusion_matrix
+
+    def indicator(self, value, label = None, trend = None, color = None, style = None):
+        self.monitor.draw_indicator(value, label = label, trend = trend, color = color, style = style) if self._no_plots else [[self._get_subplot(row, col).confusion_matrix(value, label = label, trend = trend, color = color, style = style) for col in self._Cols] for row in self._Rows]
 
 ##############################################
-###########    Build Functions    ############
-##############################################
+##############    2D Plots    ################
+############################################## 
 
-    def show(self): # it shows the current figure
-        t = time()
-        self.build()
-        ut.write(self.monitor.canvas)
-        self._time = time() - t
-        self.main()
-
-    def build(self): # it build the current figure without showing it
-        self._set_sizes()
-        self._build_matrix()
-        self.monitor.to_canvas()
-        return self.monitor.canvas
-
-    def _get_time(self, show = True): # it returns the computational time of latest show or build function
-        if show:
-            print(ut.format_time("plotext time", self._time))
-        return self._time
-
-    def save_fig(self, path = None, keep_colors = False): # it saves the plot as text or html, keep_colors = True preserves ansi colors for texts
-        path = 'plotext.txt' if path is None or not ut.correct_path(path) else path
-        _, extension = os.path.splitext(path)
-        text = self.monitor.canvas
-        if extension == ".html":
-            text = self.monitor.to_html()
-        elif not keep_colors:
-            text = ut.uncolorize(self.monitor.canvas)
-        ut.save_text(text, path)
-    savefig = save_fig
-
-##############################################
-###########    Build Utilities    ############
-##############################################
-
-    def _build_matrix(self):
-        if self._no_plots:
-            self.monitor.build_plot(*self._size) if not self.monitor.fast_plot else None
-        else:
-            [[self._get_subplot(row, col)._build_matrix() for col in self._Cols] for row in self._Rows]
-            m = [[self._get_subplot(row, col).monitor.matrix for col in self._Cols] for row in self._Rows]
-            self.monitor.matrix = m = ut.join_matrices(m)
-
-    def _take_max(self): # in a matrix of subplots the maximum height/width is considered (by default) for each row/column
-        self.max_or_min = max
-        
-    def take_min(self): # in a matrix of subplots the maximum height/width will be considered for each row/column
-        self.max_or_min = min
-
-    def _get_widths(self): # the subplots max/min widths for each column
-        widths = [[self._get_subplot(row, col)._width for row in self._Rows] for col in self._Cols]
-        widths = [self.max_or_min([sub for sub in el if sub is not None], default = None) for el in widths]
-        return widths
-
-    def _get_heights(self): # the subplots max/min heights for each row
-        heights = [[self._get_subplot(row, col)._height for col in self._Cols] for row in self._Rows]
-        heights = [self.max_or_min([sub for sub in el if sub is not None], default = None) for el in heights]
-        return heights
-
-    def _set_subplot_size(self, row = None, col = None, width = None, height = None):
-        self._get_subplot(row, col)._set_size(width, height)
-        self._get_subplot(row, col)._set_sizes() #if not self._no_plots else None
-
-    def _set_sizes(self):
-        self._set_slots_max(*self._size)
-        self._set_slots(*self._slots)
-        widths = self._get_widths()
-        widths = set_sizes(widths, self._width) # it sets the free subplots widths in accord with the parent figure width 
-        widths = fit_sizes(widths, self._width) # it fits the subplots widths to the parent figure width 
-        heights = self._get_heights()
-        heights = set_sizes(heights, self._height) # it sets the free subplots height in accord with the parent figure height 
-        heights = fit_sizes(heights, self._height) # it fits the subplots heights to the parent figure height
-        width = sum(widths) if len(widths) > 1 else self._width
-        height = sum(heights) if len(widths) > 1 else self._height
-        #self._set_size(width, height)
-        [[self._set_subplot_size(row, col, widths[col - 1], heights[row - 1]) for col in self._Cols] for row in self._Rows] if (not self._no_plots) else None
+    def matrix_plot(self, matrix, marker = None, style = None, fast = False):
+        self.monitor.draw_matrix(matrix, marker = marker, style = style, fast = fast) if self._no_plots else [[self._get_subplot(row, col).matrix_plot(matrix, marker = marker, style = style, fast = fast) for col in self._Cols] for row in self._Rows]
+    
+    def image_plot(self, path, marker = None, style = None, grayscale = False, fast = False):
+        self.monitor.draw_image(path, marker = marker, style = style, grayscale = grayscale, fast = fast) if self._no_plots else [[self._get_subplot(row, col).image_plot(path, marker = marker, style = style, grayscale = grayscale, fast = fast) for col in self._Cols] for row in self._Rows]
         
 ##############################################
 ###########    Date Functions    #############
 ##############################################
 
     def date_form(self, input_form = None, output_form = None):
-        self._master.dummy.date.date_form(input_form, output_form)
+        self._master._dummy.date.date_form(input_form, output_form)
         if self._no_plots:
             self.monitor.date.date_form(input_form, output_form)
         else:
@@ -348,7 +303,108 @@ class _figure_class():
         
     def string_to_datetime(self, string, input_form = None):
         return self.monitor.date.string_to_datetime(string, input_form = input_form)
+    
+    def string_to_time(self, string, input_form = None):
+        return self.monitor.date.string_to_time(string, input_form = input_form)
+    
+    def strings_to_time(self, string, input_form = None):
+        return self.monitor.date.strings_to_time(string, input_form = input_form)
 
+##############################################
+###########    Build Functions    ############
+##############################################
+
+    def show(self): # it build and shows the overall figure
+        t = time()
+        self.build() 
+        ut.write(self.monitor.matrix.canvas) # it prints the final canvas
+        self._time = time() - t # computational time of build + print (it does not include any pre-processing time, which is gets more important for bar and image plots)
+        self.main() if not self._master._interactive else None# it returns control to main figure on top level
+
+    def build(self): # it build the current figure without showing it
+        self._set_sizes()
+        self._build_matrix()
+        self.monitor.matrix.set_canvas() if not self.monitor.fast_plot else None
+        return self.monitor.matrix.get_canvas() 
+
+    def _build_matrix(self):
+        if self._no_plots:
+            self.monitor.build_plot() if not self.monitor.fast_plot else None
+        else:
+            [[self._get_subplot(row, col)._build_matrix() for col in self._Cols] for row in self._Rows]
+            matrices = [[self._get_subplot(row, col).monitor.matrix for col in self._Cols] for row in self._Rows]
+            self.monitor.matrix = join_matrices(matrices)
+
+##############################################
+#########    Set Size Utilities    ###########
+##############################################
+
+    def _set_sizes(self): # it properly sets coherent sub figure dimensions
+        self._set_slots_max(*self._size) 
+        self._set_slots(*self._slots)
+        widths = self._get_widths()
+        widths = ut.set_sizes(widths, self._width) # it sets the free subplots widths in accord with the parent figure width
+        widths = ut.fit_sizes(widths, self._width) # it fits the subplots widths to the parent figure width 
+        heights = self._get_heights()
+        heights = ut.set_sizes(heights, self._height) # it sets the free subplots height in accord with the parent figure height 
+        heights = ut.fit_sizes(heights, self._height) # it fits the subplots heights to the parent figure height
+        width = sum(widths) if len(widths) > 1 else self._width
+        height = sum(heights) if len(widths) > 1 else self._height
+
+        self.monitor.set_size(self._size)
+        # self._set_size(width, height)
+        [[self._set_subplot_size(row, col, widths[col - 1], heights[row - 1]) for col in self._Cols] for row in self._Rows] if (not self._no_plots) else None # to make sure that all sub figures have internal dimensions set as well 
+
+    def _get_widths(self): # the subplots max/min widths for each column
+        widths = [[self._get_subplot(row, col)._width for row in self._Rows] for col in self._Cols]
+        widths = [self.max_or_min([sub for sub in el if sub is not None], default = None) for el in widths]
+        return widths
+
+    def _get_heights(self): # the subplots max/min heights for each row
+        heights = [[self._get_subplot(row, col)._height for col in self._Cols] for row in self._Rows]
+        heights = [self.max_or_min([sub for sub in el if sub is not None], default = None) for el in heights]
+        return heights
+
+    def _set_subplot_size(self, row = None, col = None, width = None, height = None):
+        self._get_subplot(row, col)._set_size(width, height)
+        self._get_subplot(row, col)._set_sizes() 
+
+##############################################
+######    Externally Called Utility    #######
+##############################################
+
+    def _get_time(self, show = True): # it returns the computational time of latest show or build function
+        time = ut.format_time(self._time)
+        print(ut.format_strings("plotext time:", time, ut.title_color)) if show else None
+        return self._time
+
+    def main(self): # returns the master figure and sets the active figure to the master
+        self._master._active = self._master
+        return self._master
+
+    def plot_size(self, width = None, height = None):
+        width = self._width if width is None else width
+        height = self._height if height is None else height
+        self._set_size(width, height)
+        self._set_master_size() if self._is_master else None
+        self.monitor.size = self._size
+        
+    plotsize = plot_size
+
+    def take_min(self): # in a matrix of subplots the maximum height/width will be considered for each row/column
+        self.max_or_min = min
+
+    def save_fig(self, path = None, append = False, keep_colors = False): # it saves the plot as text or html, keep_colors = True preserves ansi colors for texts
+        path = 'plotext.txt' if path is None or not ut.correct_path(path) else path
+        _, extension = os.path.splitext(path)
+        canvas = self.monitor.matrix.get_canvas()
+        if extension == ".html":
+            canvas = self.monitor.matrix.to_html()
+        elif not keep_colors:
+            canvas = ut.uncolorize(canvas)
+        ut.save_text(canvas, path, append)
+    savefig = save_fig
+    
 ##############################################
 ############     Docstrings    ###############
 ##############################################
@@ -371,6 +427,8 @@ class _figure_class():
     yticks.__doc__ = doc._yticks
     xfrequency.__doc__ = doc._xfrequency
     yfrequency.__doc__ = doc._yfrequency
+    xreverse.__doc__ = doc._xreverse
+    yreverse.__doc__ = doc._yreverse
     xaxes.__doc__ = doc._xaxes
     yaxes.__doc__ = doc._yaxes
     frame.__doc__ = doc._frame
@@ -384,21 +442,28 @@ class _figure_class():
     clear_figure.__doc__ = doc._clear_figure
     clear_data.__doc__ = doc._clear_data
     clear_color.__doc__ = doc._clear_color
+    clear_terminal.__doc__ = doc._clear_terminal
     
     scatter.__doc__ = doc._scatter
     plot.__doc__ = doc._plot
-    candlestick.__doc__ = doc._candlestick
     bar.__doc__ = doc._bar
     multiple_bar.__doc__ = doc._multiple_bar
     stacked_bar.__doc__ = doc._stacked_bar
     hist.__doc__ = doc._hist
-    matrix_plot.__doc__ = doc._matrix_plot
-    image_plot.__doc__ = doc._image_plot
-    
+    candlestick.__doc__ = doc._candlestick
+
+    error.__doc__ = doc._error
     event_plot.__doc__ = doc._event_plot
     vertical_line.__doc__ = doc._vertical_line
     horizontal_line.__doc__ = doc._horizontal_line
     text.__doc__ = doc._text
+    rectangle.__doc__ = doc.rectangle
+    polygon.__doc__ = doc.polygon
+    confusion_matrix.__doc__ = doc.confusion_matrix
+    indicator.__doc__ = doc.indicator
+
+    matrix_plot.__doc__ = doc._matrix_plot
+    image_plot.__doc__ = doc._image_plot
     
     show.__doc__ = doc._show
     build.__doc__ = doc._build
@@ -411,28 +476,3 @@ class _figure_class():
     datetime_to_string.__doc__ = doc._datetime_to_string
     datetimes_to_string.__doc__ = doc._datetimes_to_string
     string_to_datetime.__doc__ = doc._string_to_datetime
-    
-##############################################
-#########    Utility Functions    ############
-##############################################
-
-def set_sizes(sizes, size_max):
-    bins = len(sizes)
-    for s in range(bins):
-        size_set = sum([el for el in sizes[0 : s] + sizes[s + 1 : ] if el is not None])
-        available = max(size_max - size_set, 0)
-        to_set = len([el for el in sizes[s : ] if el is None])
-        sizes[s] = available // to_set if sizes[s] is None else sizes[s]
-    return sizes
-
-def fit_sizes(sizes, size_max): 
-    bins = len(sizes)
-    s = bins - 1
-    #while (sum(sizes) != size_max if not_less else sum(sizes) > size_max) and s >= 0:
-    while sum(sizes) > size_max and s >= 0:
-        other_sizes = sum([sizes[i] for i in range(bins) if i != s])
-        sizes[s] = max(size_max - other_sizes, 0)
-        s -= 1
-    return sizes
-
-
